import { type PathLike } from 'fs';
import type { MarkdownHeading } from '@astrojs/markdown-remark';
export interface RenderedContent {
    /** Rendered HTML string. If present then `render(entry)` will return a component that renders this HTML. */
    html: string;
    metadata?: {
        /** Any images that are present in this entry. Relative to the {@link DataEntry} filePath. */
        imagePaths?: Array<string>;
        /** Any headings that are present in this file. */
        headings?: MarkdownHeading[];
        /** Raw frontmatter, parsed parsed from the file. This may include data from remark plugins. */
        frontmatter?: Record<string, any>;
        /** Any other metadata that is present in this file. */
        [key: string]: unknown;
    };
}
export interface DataEntry<TData extends Record<string, unknown> = Record<string, unknown>> {
    /** The ID of the entry. Unique per collection. */
    id: string;
    /** The parsed entry data */
    data: TData;
    /** The file path of the content, if applicable. Relative to the site root. */
    filePath?: string;
    /** The raw body of the content, if applicable. */
    body?: string;
    /** An optional content digest, to check if the content has changed. */
    digest?: number | string;
    /** The rendered content of the entry, if applicable. */
    rendered?: RenderedContent;
    /**
     * If an entry is a deferred, its rendering phase is delegated to a virtual module during the runtime phase when calling `renderEntry`.
     */
    deferredRender?: boolean;
}
export declare class DataStore {
    #private;
    constructor();
    get<T = unknown>(collectionName: string, key: string): T | undefined;
    entries<T = unknown>(collectionName: string): Array<[id: string, T]>;
    values<T = unknown>(collectionName: string): Array<T>;
    keys(collectionName: string): Array<string>;
    set(collectionName: string, key: string, value: unknown): void;
    delete(collectionName: string, key: string): void;
    clear(collectionName: string): void;
    clearAll(): void;
    has(collectionName: string, key: string): boolean;
    hasCollection(collectionName: string): boolean;
    collections(): Map<string, Map<string, any>>;
    addAssetImport(assetImport: string, filePath: string): void;
    addAssetImports(assets: Array<string>, filePath: string): void;
    addModuleImport(fileName: string): void;
    writeAssetImports(filePath: PathLike): Promise<void>;
    writeModuleImports(filePath: PathLike): Promise<void>;
    scopedStore(collectionName: string): ScopedDataStore;
    /**
     * Returns a MetaStore for a given collection, or if no collection is provided, the default meta collection.
     */
    metaStore(collectionName?: string): MetaStore;
    toString(): string;
    writeToDisk(filePath: PathLike): Promise<void>;
    /**
     * Attempts to load a DataStore from the virtual module.
     * This only works in Vite.
     */
    static fromModule(): Promise<DataStore>;
    static fromMap(data: Map<string, Map<string, any>>): Promise<DataStore>;
    static fromString(data: string): Promise<DataStore>;
    static fromFile(filePath: string | URL): Promise<DataStore>;
}
export interface ScopedDataStore {
    get: <TData extends Record<string, unknown> = Record<string, unknown>>(key: string) => DataEntry<TData> | undefined;
    entries: () => Array<[id: string, DataEntry]>;
    set: <TData extends Record<string, unknown>>(opts: {
        /** The ID of the entry. Must be unique per collection. */
        id: string;
        /** The data to store. */
        data: TData;
        /** The raw body of the content, if applicable. */
        body?: string;
        /** The file path of the content, if applicable. Relative to the site root. */
        filePath?: string;
        /** A content digest, to check if the content has changed. */
        digest?: number | string;
        /** The rendered content, if applicable. */
        rendered?: RenderedContent;
        /**
         * If an entry is a deferred, its rendering phase is delegated to a virtual module during the runtime phase.
         */
        deferredRender?: boolean;
    }) => boolean;
    values: () => Array<DataEntry>;
    keys: () => Array<string>;
    delete: (key: string) => void;
    clear: () => void;
    has: (key: string) => boolean;
    /**
     * Adds a single asset to the store. This asset will be transformed
     * by Vite, and the URL will be available in the final build.
     * @param fileName
     * @param specifier
     * @returns
     */
    addModuleImport: (fileName: string) => void;
}
/**
 * A key-value store for metadata strings. Useful for storing things like sync tokens.
 */
export interface MetaStore {
    get: (key: string) => string | undefined;
    set: (key: string, value: string) => void;
    has: (key: string) => boolean;
    delete: (key: string) => void;
}
export declare function contentModuleToId(fileName: string): string;
